
/* Copyright 2008-2019 Douglas Wikstrom
 *
 * This file is part of Verificatum Mix-Net (VMN).
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.verificatum.protocol.hvzk;

import com.verificatum.eio.ByteTreeBasic;
import com.verificatum.protocol.coinflip.CoinFlipPRingSource;
import com.verificatum.ui.Log;

/**
 * Container class for a coin-flipping functionality used to generate
 * challenges for public coin protocols.
 *
 * @author Douglas Wikstrom
 */
public final class ChallengerI implements Challenger {

    /**
     * Source of jointly generated random coins.
     */
    CoinFlipPRingSource coins;

    /**
     * Creates an instance which generates challenges using the given
     * source of jointly generated random coins.
     *
     * @param coins Source of jointly generated random coins.
     */
    public ChallengerI(final CoinFlipPRingSource coins) {
        this.coins = coins;
    }

    // Documented in Challenger.java

    @Override
    public byte[] challenge(final Log log, final ByteTreeBasic data,
                            final int vbitlen, final int rbitlen) {

        log.info("Generate bits jointly.");
        final Log tempLog = log.newChildLog();

        return coins.getCoinBytes(tempLog, vbitlen, rbitlen);
    }
}
